/*
 * Szegedi Tudományegyetem
 * Informatikai Tanszékcsoport
 * Szoftverfejlesztés Tanszék
 */

#include "pnm-manipulation.h"
#include "pnm-type.h"

#include <dlfcn.h>
#include <stdio.h>
#include <string.h>

typedef struct modparam {
    char *option;
    char *name;
} modparam;

modparam otable[] = {
    {"-i",   "rgbpic_invert"},
    {"-iR",  "rgbpic_invert_R"},
    {"-iG",  "rgbpic_invert_G"},
    {"-iB",  "rgbpic_invert_B"},
    {"-v",   "rgbpic_mirror_v"},
    {"-h",   "rgbpic_mirror_h"},
    {"-c",   "rgbpic_mirror_c"},
    {"-l",   "rgbpic_rotate_l"},
    {"-r",   "rgbpic_rotate_r"},
    {"-xRG", "rgbpic_color_RG"},
    {"-xGB", "rgbpic_color_GB"},
    {"-xBR", "rgbpic_color_BR"},
    {"-dR",  "rgbpic_delete_R"},
    {"-dG",  "rgbpic_delete_G"},
    {"-dB",  "rgbpic_delete_B"},
    {"-g",   "rgbpic_convert_to_grey_lum"},
    {"-gR",  "rgbpic_convert_R_to_grey"},
    {"-gG",  "rgbpic_convert_G_to_grey"},
    {"-gB",  "rgbpic_convert_B_to_grey"},
    {"-b",   "rgbpic_convert_to_bw_lum"},
    {NULL, NULL}
};

typedef int (*savf)(const char*, rgbpic_t*);
typedef struct savparam {
    char *option;
    savf function;
} savparam;

savparam stable[] = {
    {"-PBM", rgbpic_save_PBM},
    {"-PGM", rgbpic_save_PGM},
    {"-PPM", rgbpic_save_PPM},
    {NULL, NULL}
};

/*
 * A main függvény.
 */
int main(int argc, char *argv[]) {
    int argi;
    rgbpic_t *kep;                              /* A kép (illetve a képre mutató pointer) */
    void *lib_handle;
    char *error_msg;

    if (argc < 5) {                                            /* Ha nincs elég paraméter */
        printf("Nincs eleg parameter!\n");
        printf("Használat: %s <dynamic-library> <inputfile>", argv[0]);
        if (stable->option) {
            savparam *sp = stable;
            printf(" %s", sp->option);
            while ((++sp)->option) {
                printf("|%s", sp->option);
            }
        }
        printf(" <outputfile>");
        if (otable->option) {
            modparam *mp = otable;
            printf(" [%s", mp->option);
            while ((++mp)->option) {
                printf(", %s", mp->option);
            }
        }
        printf("]\n");
        return 1;
    }

    lib_handle = dlopen(argv[1], RTLD_LAZY);
    if (!lib_handle) {
        fprintf(stderr, "Nem sikerült megnyitni a %s lib-et!\n%s\n", argv[1], dlerror());
        return 1;
    }

    if (!(kep = rgbpic_load(argv[2]))) {  /* Az első paramétert input képként értelmezzük */
        printf("Nem sikerult megnyitni: %s\n", argv[2]);
        dlclose(lib_handle);
        return 1;
    }

    /*
     * A 4. paramétertől kezdve az elvégzendő műveletek kapcsolói
     * szerepelnek.
     */
    for (argi = 5; argi < argc; ++argi) {
        /*
         * Megkeressük, hogy a következő kapcsoló benne van-e az
         * előbb összeállított táblázatunkban.
         */
        for (modparam *p = otable; p->option; ++p) {
            /*
             * Ha megtaláltuk, a hozzá tartozó módosító
             * függvényt végrehajtjuk a képre, és nem
             * keressük tovább, hanem nézzük a következő
             * paramétert.
             */
            if (!strcmp(p->option, argv[argi])) {
                rgbpic_manipulation_fnc_t function;
                function = dlsym(lib_handle, p->name);
                if ((error_msg = dlerror()) != NULL) {
                    fprintf(stderr, "%s\n", error_msg);
                    break;
                }
                function(kep);
                break;
            }
        }
    }

    dlclose(lib_handle);
    /*
     * A 2. paraméter a kiíratás formátumát adja meg, ezt is
     * megkeressük az előre megadott másik tömbünkben.
     */
    for (savparam *sp = stable; sp->option; ++sp) {
        if (!strcmp(sp->option, argv[3])) {
            /*
             * Ha megtaláltuk, akkor a hozzá tartozó
             * függvény segítségével elmentjük a képet a
             * megadott néven (3. paraméter).
             */
            if (sp->function(argv[4], kep)) {
                /* Ha a mentés nem sikerült, a program
                 * szövegesen is jelez és hibakóddal tér
                 * vissza.
                 */
                printf("Nem sikerult elmenteni: %s\n", argv[3]);
                rgbpic_delete(kep);
                return 1;
            }
            break;
        }
    }
    /* Töröljük a képet. */
    rgbpic_delete(kep);
    return 0;
}
